@@ -16,13 +16,13 @@ import pandas as pd
 import requests as requests
 from typing_extensions import assert_never
 
-from chalk import EnvironmentId
 from chalk.client import ChalkClient, Dataset, DatasetRevision
 from chalk.client.models import (
     ChalkError,
     ColumnMetadata,
     DatasetFilter,
     DatasetResponse,
+    DatasetRevisionResponse,
     OfflineQueryContext,
     QueryStatus,
 )
@@ -30,6 +30,7 @@ from chalk.features import DataFrame, Feature, FeatureWrapper, deserialize_dtype
 from chalk.features._encoding.pyarrow import pyarrow_to_polars
 from chalk.features.feature_set import FeatureSetBase
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE, ID_FEATURE, OBSERVED_AT_FEATURE, PSEUDONAMESPACE
+from chalk.features.tag import BranchId, EnvironmentId
 from chalk.utils.collections import get_unique_item
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -63,21 +64,25 @@ class ColNameDecoder:
 class DatasetVersion(IntEnum):
     """Format of the parquet file. Used when loading a dataset so that we know what format it is in"""
 
-    # This is the format that bigquery dumps to when specifying an output bucket and output format
-    # as part of an (async) query job
-    # The output contains extra columns, and all column names are b32 encoded, because
-    # bigquery does not support '.' in column names.
-    # The client will have to decode column names before loading this data
-    # All data, except for feature times, are json encoded
     BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES = 1
+    """
+    This is the format that bigquery dumps to when specifying an output bucket and output format
+    as part of an (async) query job
+    The output contains extra columns, and all column names are b32 encoded, because
+    bigquery does not support '.' in column names.
+    The client will have to decode column names before loading this data
+    All data, except for feature times, are json encoded
+    """
 
-    # This is the format returned by the dataset writer in engine/
     DATASET_WRITER = 2
+    """This is the format returned by the dataset writer in engine/"""
 
-    # This format uses separate columns for the observed at and timestamp columns
-    # The observed at column is the actual timestamp from when the observation was observed,
-    # whereas the timestamp column is the original timestamp that the user requested
     BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2 = 3
+    """
+    This format uses separate columns for the observed at and timestamp columns
+    The observed at column is the actual timestamp from when the observation was observed,
+    whereas the timestamp column is the original timestamp that the user requested
+    """
 
     COMPUTE_RESOLVER_OUTPUT_V1 = 4
 
@@ -438,6 +443,7 @@ class DatasetRevisionImpl(DatasetRevision):
         terminated_at: Optional[datetime] = None,
         dataset_name: Optional[str] = None,
         dataset_id: Optional[uuid.UUID] = None,
+        branch: Optional[str] = None,
     ):
         self.revision_id = revision_id
         self.environment = environment
@@ -456,6 +462,7 @@ class DatasetRevisionImpl(DatasetRevision):
         self.dataset_name = dataset_name
         self.dataset_id = dataset_id
         self._client = client
+        self._branch: BranchId = branch
 
     @property
     def data_as_polars(self) -> pl.LazyFrame:
@@ -480,6 +487,7 @@ class DatasetRevisionImpl(DatasetRevision):
             context=context,
             output_id=output_id,
             output_ts=output_ts,
+            branch=self._branch,
         ).lazy()
 
     @property
@@ -506,6 +514,7 @@ class DatasetRevisionImpl(DatasetRevision):
                 context=context,
                 output_id=output_id,
                 output_ts=output_ts,
+                branch=self._branch,
             )
             .lazy()
             .collect()
@@ -536,6 +545,7 @@ class DatasetRevisionImpl(DatasetRevision):
                 context=context,
                 output_id=output_id,
                 output_ts=output_ts,
+                branch=self._branch,
             )
         )
 
@@ -555,6 +565,7 @@ class DatasetRevisionImpl(DatasetRevision):
             urls_only=True,
             output_id=output_id,
             output_ts=output_ts,
+            branch=self._branch,
         )
         if num_executors:
             executor = ThreadPoolExecutor(16)
@@ -585,49 +596,21 @@ class DatasetImpl(Dataset):
         is_finished: bool,
         version: int,
         revisions: List[DatasetRevisionImpl],
-        client: ChalkClient,
+        client: ChalkAPIClientImpl,
         environment: EnvironmentId,
         dataset_id: Optional[uuid.UUID] = None,
         dataset_name: Optional[str] = None,
         errors: Optional[List[ChalkError]] = None,
     ):
-        self._is_finished = is_finished
-        self._version = version
-        self._revisions = revisions
-        self._environment = environment
-        self._dataset_id = dataset_id
-        self._dataset_name = dataset_name
-        self._errors = errors
+        self.is_finished = is_finished
+        self.version = version
+        self.revisions = revisions
+        self.environment = environment
+        self.dataset_id = dataset_id
+        self.dataset_name = dataset_name
+        self.errors = errors
         self._client = client
 
-    @property
-    def is_finished(self) -> bool:
-        return self._is_finished
-
-    @property
-    def version(self) -> int:
-        return self._version
-
-    @property
-    def revisions(self) -> Sequence[DatasetRevisionImpl]:
-        return self._revisions
-
-    @property
-    def environment(self) -> EnvironmentId:
-        return self._environment
-
-    @property
-    def dataset_id(self) -> Union[uuid.UUID, None]:
-        return self._dataset_id
-
-    @property
-    def dataset_name(self) -> Union[str, None]:
-        return self._dataset_name
-
-    @property
-    def errors(self) -> Optional[List[ChalkError]]:
-        return self._errors
-
     @property
     def data_as_polars(self) -> pl.LazyFrame:
         warnings.warn(
@@ -642,9 +625,9 @@ class DatasetImpl(Dataset):
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pl.LazyFrame:
-        if len(self._revisions) == 0:
+        if len(self.revisions) == 0:
             raise IndexError("No revisions exist for dataset")
-        return self._revisions[-1].get_data_as_polars(
+        return self.revisions[-1].get_data_as_polars(
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -663,9 +646,9 @@ class DatasetImpl(Dataset):
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pd.DataFrame:
-        if len(self._revisions) == 0:
+        if len(self.revisions) == 0:
             raise IndexError("No revisions exist for dataset")
-        return self._revisions[-1].get_data_as_pandas(
+        return self.revisions[-1].get_data_as_pandas(
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -684,9 +667,9 @@ class DatasetImpl(Dataset):
         output_id: bool = False,
         output_ts: bool = False,
     ) -> DataFrame:
-        if len(self._revisions) == 0:
+        if len(self.revisions) == 0:
             raise IndexError("No revisions exist for dataset")
-        return self._revisions[-1].get_data_as_dataframe(
+        return self.revisions[-1].get_data_as_dataframe(
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -695,43 +678,59 @@ class DatasetImpl(Dataset):
         self,
         path: str,
         num_executors: Optional[int] = 16,
-    ):
-        return self._revisions[-1].download_data(
+    ) -> None:
+        return self.revisions[-1].download_data(
             path=path,
             num_executors=num_executors,
         )
 
+    def recompute(self, branch: BranchId, features: Optional[List[Union[Feature, Any]]] = None):
+        new_revision = self._client._recompute_dataset(
+            dataset_name=self.dataset_name,
+            dataset_id=self.dataset_id,
+            revision_id=self.revisions[-1].revision_id,
+            features=features or [],
+            branch=branch,
+            environment=self.environment,
+        )
+        self.revisions.append(dataset_revision_from_response(new_revision, self._client))
+        return self
+
     def __repr__(self) -> str:
-        if self._errors:
-            return f"Dataset(name='{self._dataset_name}', version='{self._version}', errors='{self._errors}')"
-        if self._dataset_name:
-            return f"Dataset(name='{self._dataset_name}', version='{self._version}')"
+        if self.errors:
+            return f"Dataset(name='{self.dataset_name}', version='{self.version}', errors='{self.errors}')"
+        if self.dataset_name:
+            return f"Dataset(name='{self.dataset_name}', version='{self.version}')"
         return f"Dataset()"
 
 
+def dataset_revision_from_response(
+    revision: DatasetRevisionResponse, client: ChalkAPIClientImpl
+) -> DatasetRevisionImpl:
+    return DatasetRevisionImpl(
+        revision_id=revision.revision_id,
+        environment=revision.environment_id,
+        creator_id=revision.creator_id,
+        outputs=revision.outputs,
+        givens_uri=revision.givens_uri,
+        status=revision.status,
+        filters=revision.filters,
+        num_partitions=revision.num_partitions,
+        output_uris=revision.output_uris,
+        output_version=revision.output_version,
+        num_bytes=revision.num_bytes,
+        client=client,
+        created_at=revision.created_at,
+        started_at=revision.started_at,
+        terminated_at=revision.terminated_at,
+        dataset_name=revision.dataset_name,
+        dataset_id=revision.dataset_id,
+        branch=revision.branch,
+    )
+
+
 def dataset_from_response(response: DatasetResponse, client: ChalkAPIClientImpl) -> DatasetImpl:
-    revisions = [
-        DatasetRevisionImpl(
-            revision_id=revision.revision_id,
-            environment=revision.environment_id,
-            creator_id=revision.creator_id,
-            outputs=revision.outputs,
-            givens_uri=revision.givens_uri,
-            status=revision.status,
-            filters=revision.filters,
-            num_partitions=revision.num_partitions,
-            output_uris=revision.output_uris,
-            output_version=revision.output_version,
-            num_bytes=revision.num_bytes,
-            client=client,
-            created_at=revision.created_at,
-            started_at=revision.started_at,
-            terminated_at=revision.terminated_at,
-            dataset_name=revision.dataset_name,
-            dataset_id=revision.dataset_id,
-        )
-        for revision in response.revisions
-    ]
+    revisions = [dataset_revision_from_response(revision, client) for revision in response.revisions]
     return DatasetImpl(
         is_finished=response.is_finished,
         version=response.version,
