@@ -8,7 +8,7 @@ from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Union
 import numpy as np
 from pydantic import BaseModel, Field, root_validator
 
-from chalk import EnvironmentId
+from chalk.features.tag import EnvironmentId
 
 
 def _category_for_error_code(c: Union[ErrorCode, str]) -> ErrorCodeCategory:
@@ -439,6 +439,14 @@ class ComputeResolverOutputRequest(BaseModel):
     environment: Optional[str] = None
 
 
+class DatasetRecomputeRequest(BaseModel):
+    dataset_name: Optional[str] = None
+    branch: str
+    dataset_id: Optional[str]
+    revision_id: Optional[str]
+    features: List[str]
+
+
 class RecomputeResolverOutputRequest(BaseModel):
     persistent_id: str
     resolver_fqn: str
@@ -557,6 +565,7 @@ class DatasetRevisionResponse(BaseModel):
     num_bytes: Optional[int] = None
     output_uris: str
     output_version: int
+    branch: Optional[str] = None
 
 
 class DatasetResponse(BaseModel):
@@ -573,3 +582,11 @@ class DatasetResponse(BaseModel):
     dataset_name: Optional[str] = None
     revisions: List[DatasetRevisionResponse]
     errors: Optional[List[ChalkError]] = None
+
+
+class UpdateResolverResponse(BaseModel):
+    updated_fqn: Optional[
+        str
+    ] = None  # The resolver fqn that was updated (may not be the same as the one that was requested)
+    is_new: Optional[bool] = None  # Whether a new resolver was created, or if an existing one was replaced
+    errors: Optional[List[ChalkError]] = None
