@@ -241,7 +241,7 @@ def _structure_basic(obj: Any, typ: Type):
         raise TypeError(f"Cannot convert '{obj}' to a Boolean. Valid values are 1, True, 0, or False.")
     if issubclass(typ, (int, float)):
         if typ(obj) != obj:
-            raise TypeError(f"Cannot cast '{obj}' to a {typ} without loosing precision")
+            raise TypeError(f"Cannot cast '{obj}' to a {typ} without losing precision")
     return typ(obj)
 
 
