@@ -543,10 +543,11 @@ def _process_class(
             raise TypeError(f"Windowed feature '{namespace}.{name}' is missing an annotation, like 'Windowed[str]'")
         if name not in cls_annotations and isinstance(member, Feature):
             # All feature types need annotations, except for datetimes, which we can automatically infer
-            if member.typ is not None and member.typ.is_parsed:
-                cls_annotations[name] = member.typ.annotation
-            else:
-                raise TypeError(f"Feature '{namespace}.{name}' is missing an annotation")
+            if not member.is_typ_set() or not member.typ.is_parsed:
+                raise TypeError(
+                    f"Feature '{namespace}.{name}' is missing an annotation. Please add one, like '{name}: str = ...'"
+                )
+            cls_annotations[name] = member.typ.annotation
 
     cls.__annotations__ = cls_annotations
 
