@@ -2,7 +2,9 @@ from __future__ import annotations
 
 import abc
 import dataclasses
+import datetime
 import inspect
+import random
 from dataclasses import dataclass
 from inspect import Parameter, isclass
 from typing import (
@@ -25,10 +27,12 @@ from typing import (
     overload,
 )
 
+import cloudpickle
 import pyarrow
 from pydantic import BaseModel
-from typing_extensions import ParamSpec, get_args, get_origin
+from typing_extensions import ParamSpec, TypeAlias, get_args, get_origin
 
+from chalk.client.client_impl import ChalkAPIClientImpl
 from chalk.features.dataframe import DataFrame, DataFrameMeta
 from chalk.features.feature_field import Feature
 from chalk.features.feature_set import Features
@@ -45,11 +49,12 @@ from chalk.streams.types import (
     StreamResolverParamMessageWindow,
     StreamResolverSignature,
 )
-from chalk.utils import MachineType
+from chalk.utils import MachineType, notebook
 from chalk.utils.annotation_parsing import ResolverAnnotationParser
 from chalk.utils.collections import ensure_tuple
 from chalk.utils.duration import CronTab, Duration, parse_chalk_duration
 from chalk.utils.environment_parsing import env_var_bool
+from chalk.utils.log_with_context import get_logger
 
 try:
     from types import UnionType
@@ -64,6 +69,10 @@ T_co = TypeVar("T_co", covariant=True)
 P = ParamSpec("P")
 V = TypeVar("V")
 
+ResolverHook: TypeAlias = "ClassVar[Optional[Callable[[Resolver], None]]]"
+
+_logger = get_logger(__name__)
+
 
 @dataclasses.dataclass(frozen=True)
 class ResolverArgErrorHandler:
@@ -195,7 +204,28 @@ class Resolver(ResolverProtocol[P, T], abc.ABC):
     default_args: List[Optional[ResolverArgErrorHandler]]
 
     registry: "ClassVar[List[Resolver]]" = []
-    hook: "ClassVar[Optional[Callable[[Resolver], None]]]" = None
+
+    @classmethod
+    def _deploy_on_notebook_change(cls, resolver: Resolver):
+        """
+        If this code is being run in a Jupyter notebook, cloudpickle this
+        resolver's dependencies and send it to the API server.
+        """
+        if not notebook.is_notebook():
+            return
+        pickled_resolver = cloudpickle.dumps(resolver)
+        client = ChalkAPIClientImpl._latest_client
+        # TODO ignore errors for now until server endpoint is created
+        try:
+            resp = client._send_updated_resolver(resolver.environment, pickled_resolver)
+            if resp.errors:
+                for e in resp.errors:
+                    _logger.debug(f"{e}")
+                raise RuntimeError("Failed to update resolvers.")
+        except Exception:
+            pass
+
+    hook: ResolverHook = _deploy_on_notebook_change
 
     def __eq__(self, other: object):
         if not isinstance(other, type(self)):
@@ -842,7 +872,11 @@ def online(
 
     def decorator(fn: Callable[P, T]):
         parsed = parse_function(fn, caller_globals, caller_locals)
-        if not env_var_bool("CHALK_ALLOW_REGISTRY_UPDATES") and parsed.fqn in {s.fqn for s in Resolver.registry}:
+        if (
+            not env_var_bool("CHALK_ALLOW_REGISTRY_UPDATES")
+            and parsed.fqn in {s.fqn for s in Resolver.registry}
+            and not notebook.is_notebook()
+        ):
             raise ValueError(f"Duplicate resolver {parsed.fqn}")
         if parsed.output is None:
             raise ValueError(f"Online resolvers must return features; '{parsed.fqn}' returns None")
@@ -1003,7 +1037,11 @@ def offline(
 
     def decorator(fn: Callable[P, T]):
         parsed = parse_function(fn, caller_globals, caller_locals)
-        if not env_var_bool("CHALK_ALLOW_REGISTRY_UPDATES") and parsed.fqn in {s.fqn for s in Resolver.registry}:
+        if (
+            not env_var_bool("CHALK_ALLOW_REGISTRY_UPDATES")
+            and parsed.fqn in {s.fqn for s in Resolver.registry}
+            and not notebook.is_notebook()
+        ):
             raise ValueError(f"Duplicate resolver {parsed.fqn}")
         if parsed.output is None:
             raise ValueError(f"Offline resolvers must return features; '{parsed.fqn}' returns None")
