@@ -62,7 +62,7 @@ class VersionInfoGQL:
 @dataclass
 class UpsertScalarKindGQL:
     primary: bool
-    dtype: Optional[Dict[str, Any]] = None  # The dict form of the chalk.features.SerializedDType model
+    dtype: Optional[str] = None  # The JSON-serialized form of the chalk.features.SerializedDType model
     version: Optional[int] = None  # Deprecated. Use the `versionInfo` instead
     versionInfo: Optional[VersionInfoGQL] = None
     baseClasses: Optional[List[str]] = None  # Deprecated. Use the `dtype` instead
@@ -157,6 +157,12 @@ class UpsertResolverOutputGQL:
     dataframes: Optional[List[UpsertDataFrameGQL]] = None
 
 
+@dataclass
+class UpsertResolverInputUnionGQL:
+    feature: Optional[UpsertFeatureReferenceGQL] = None
+    dataframe: Optional[UpsertDataFrameGQL] = None
+
+
 @dataclass
 class UpsertResolverGQL:
     fqn: str
@@ -168,6 +174,7 @@ class UpsertResolverGQL:
     doc: Optional[str] = None
     cron: Optional[str] = None
     inputs: Optional[List[UpsertFeatureReferenceGQL]] = None
+    all_inputs: Optional[List[UpsertResolverInputUnionGQL]] = None
     machineType: Optional[str] = None
     owner: Optional[str] = None
     timeout: Optional[str] = None
