@@ -1,5 +1,5 @@
 from os import PathLike
-from typing import Optional, Union, overload
+from typing import Any, Dict, Optional, Union, overload
 
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
@@ -8,7 +8,7 @@ from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
 from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
 from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
 from chalk.sql._internal.integrations.snowflake import SnowflakeSourceImpl
-from chalk.sql._internal.integrations.sqlite import SQLiteFileSourceImpl, SQLiteInMemorySourceImpl
+from chalk.sql._internal.integrations.sqlite import SQLiteSourceImpl
 from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import (
     BaseSQLSourceProtocol,
@@ -39,7 +39,7 @@ def SnowflakeSource() -> BaseSQLSourceProtocol:
 
 
 @overload
-def SnowflakeSource(*, name: str) -> BaseSQLSourceProtocol:
+def SnowflakeSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -> BaseSQLSourceProtocol:
     """Chalk's injects environment variables to support data integrations.
 
     But what happens when you have two data sources of the same kind?
@@ -51,6 +51,9 @@ def SnowflakeSource(*, name: str) -> BaseSQLSourceProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -74,6 +77,7 @@ def SnowflakeSource(
     db: str = ...,
     schema: str = ...,
     role: str = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -95,6 +99,8 @@ def SnowflakeSource(
         Snowflake schema in the database to use.
     role
         Snowflake role name to use.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -127,6 +133,7 @@ def SnowflakeSource(
     db: Optional[str] = None,
     schema: Optional[str] = None,
     role: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> BaseSQLSourceProtocol:
     """Create a Snowflake data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -146,6 +153,7 @@ def SnowflakeSource(
         db=db,
         schema=schema,
         role=role,
+        engine_args=engine_args,
     )
 
 
@@ -170,7 +178,11 @@ def PostgreSQLSource() -> SQLSourceWithTableIngestProtocol:
 
 
 @overload
-def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
+def PostgreSQLSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> SQLSourceWithTableIngestProtocol:
     """If you have only one PostgreSQL integration, there's no need to provide
     a distinguishing name.
 
@@ -183,6 +195,9 @@ def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -204,6 +219,7 @@ def PostgreSQLSource(
     db: str = ...,
     user: str = ...,
     password: str = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> SQLSourceWithTableIngestProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -221,6 +237,8 @@ def PostgreSQLSource(
         PostgreSQL username to connect as.
     password
         The password to be used if the server demands password authentication.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -253,6 +271,7 @@ def PostgreSQLSource(
     user: Optional[str] = None,
     password: Optional[str] = None,
     name: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> TableIngestProtocol:
     """Create a PostgreSQL data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -263,7 +282,7 @@ def PostgreSQLSource(
     the data source. See the overloaded signatures for more
     details.
     """
-    return PostgreSQLSourceImpl(host, port, db, user, password, name)
+    return PostgreSQLSourceImpl(host, port, db, user, password, name, engine_args=engine_args)
 
 
 @overload
@@ -285,7 +304,11 @@ def MySQLSource() -> SQLSourceWithTableIngestProtocol:
 
 
 @overload
-def MySQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
+def MySQLSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> SQLSourceWithTableIngestProtocol:
     """If you have only one MySQL integration, there's no need to provide
     a distinguishing name.
 
@@ -298,6 +321,9 @@ def MySQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -319,6 +345,7 @@ def MySQLSource(
     db: str = ...,
     user: str = ...,
     password: str = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> SQLSourceWithTableIngestProtocol:
     """
     You can also configure the integration directly using environment
@@ -337,6 +364,8 @@ def MySQLSource(
         MySQL username to connect as.
     password
         The password to be used if the server demands password authentication.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -369,6 +398,7 @@ def MySQLSource(
     user: Optional[str] = None,
     password: Optional[str] = None,
     name: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> SQLSourceWithTableIngestProtocol:
     """Create a MySQL data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -379,10 +409,12 @@ def MySQLSource(
     the data source. See the overloaded signatures for more
     details.
     """
-    return MySQLSourceImpl(host, port, db, user, password, name)
+    return MySQLSourceImpl(host, port, db, user, password, name, engine_args=engine_args)
 
 
-def SQLiteInMemorySource(name: Optional[str] = None) -> SQLSourceWithTableIngestProtocol:
+def SQLiteInMemorySource(
+    name: Optional[str] = None, engine_args: Optional[Dict[str, Any]] = None
+) -> SQLSourceWithTableIngestProtocol:
     """Testing SQL source.
 
     If you have only one SQLiteInMemorySource integration, there's no need to provide
@@ -392,6 +424,8 @@ def SQLiteInMemorySource(name: Optional[str] = None) -> SQLSourceWithTableIngest
     ----------
     name
         The name of the integration.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -400,12 +434,14 @@ def SQLiteInMemorySource(name: Optional[str] = None) -> SQLSourceWithTableIngest
 
     Examples
     --------
-    >>> source = MySQLSource(name="RISK")
+    >>> source = SQLiteInMemorySource(name="RISK")
     """
-    return SQLiteInMemorySourceImpl(name)
+    return SQLiteSourceImpl(name=name, engine_args=engine_args)
 
 
-def SQLiteFileSource(filename: Union[str, PathLike], name: Optional[str] = None) -> SQLSourceWithTableIngestProtocol:
+def SQLiteFileSource(
+    filename: Union[str, PathLike], name: Optional[str] = None, engine_args: Optional[Dict[str, Any]] = None
+) -> SQLSourceWithTableIngestProtocol:
     """Create a SQLite source for a file.
 
     Parameters
@@ -414,13 +450,15 @@ def SQLiteFileSource(filename: Union[str, PathLike], name: Optional[str] = None)
         The name of the file.
     name
         The name to use in testing
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
     SQLSourceWithTableIngestProtocol
         The SQL source for use in Chalk resolvers.
     """
-    return SQLiteFileSourceImpl(filename, name)
+    return SQLiteSourceImpl(filename=filename, name=name, engine_args=engine_args)
 
 
 @overload
@@ -442,7 +480,11 @@ def RedshiftSource() -> BaseSQLSourceProtocol:
 
 
 @overload
-def RedshiftSource(*, name: str) -> BaseSQLSourceProtocol:
+def RedshiftSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
     """If you have only one Redshift integration, there's no need to provide
     a distinguishing name.
 
@@ -455,6 +497,9 @@ def RedshiftSource(*, name: str) -> BaseSQLSourceProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -475,6 +520,7 @@ def RedshiftSource(
     db: str = ...,
     user: str = ...,
     password: str = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -490,6 +536,8 @@ def RedshiftSource(
         Redshify username to connect as.
     password
         The password for the Redshift database.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -520,6 +568,7 @@ def RedshiftSource(
     user: Optional[str] = None,
     password: Optional[str] = None,
     name: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> BaseSQLSourceProtocol:
     """Create a Redshift data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -530,7 +579,7 @@ def RedshiftSource(
     the data source. See the overloaded signatures for more
     details.
     """
-    return RedshiftSourceImpl(host, db, user, password, name)
+    return RedshiftSourceImpl(host, db, user, password, name, engine_args=engine_args)
 
 
 @overload
@@ -552,7 +601,11 @@ def BigQuerySource() -> BaseSQLSourceProtocol:
 
 
 @overload
-def BigQuerySource(*, name: str) -> BaseSQLSourceProtocol:
+def BigQuerySource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
     """If you have only one BigQuery integration, there's no need to provide
     a distinguishing name.
 
@@ -565,6 +618,9 @@ def BigQuerySource(*, name: str) -> BaseSQLSourceProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -581,11 +637,12 @@ def BigQuerySource(*, name: str) -> BaseSQLSourceProtocol:
 @overload
 def BigQuerySource(
     *,
-    project: str = None,
-    dataset: str = None,
-    location: str = None,
-    credentials_base64: Optional[str] = None,
-    credentials_path: Optional[str] = None,
+    project: Optional[str] = ...,
+    dataset: Optional[str] = ...,
+    location: Optional[str] = ...,
+    credentials_base64: Optional[str] = ...,
+    credentials_path: Optional[str] = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -603,6 +660,8 @@ def BigQuerySource(
         The credentials to use to connect, encoded as a base64 string.
     credentials_path
         The path to the credentials file to use to connect.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -630,6 +689,7 @@ def BigQuerySource(
     location: Optional[str] = None,
     credentials_base64: Optional[str] = None,
     credentials_path: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> BaseSQLSourceProtocol:
     """Create a BigQuery data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -647,6 +707,7 @@ def BigQuerySource(
         location=location,
         credentials_base64=credentials_base64,
         credentials_path=credentials_path,
+        engine_args=engine_args,
     )
 
 
@@ -669,7 +730,11 @@ def CloudSQLSource() -> BaseSQLSourceProtocol:
 
 
 @overload
-def CloudSQLSource(*, name: str) -> BaseSQLSourceProtocol:
+def CloudSQLSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+) -> BaseSQLSourceProtocol:
     """If you have only one CloudSQL integration, there's no need to provide
     a distinguishing name.
 
@@ -682,6 +747,9 @@ def CloudSQLSource(*, name: str) -> BaseSQLSourceProtocol:
     ----------
     name
         Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
 
     Returns
     -------
@@ -698,10 +766,11 @@ def CloudSQLSource(*, name: str) -> BaseSQLSourceProtocol:
 @overload
 def CloudSQLSource(
     *,
-    instance_name: str = None,
-    db: str = None,
-    user: str = None,
-    password: str = None,
+    instance_name: Optional[str] = ...,
+    db: Optional[str] = ...,
+    user: Optional[str] = ...,
+    password: Optional[str] = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -717,6 +786,8 @@ def CloudSQLSource(
         Username to use.
     password
         The password to use.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
 
     Returns
     -------
@@ -742,6 +813,7 @@ def CloudSQLSource(
     db: Optional[str] = None,
     user: Optional[str] = None,
     password: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
 ) -> BaseSQLSourceProtocol:
     """Create a CloudSQL data source. SQL-based data sources
     created without arguments assume a configuration in your
@@ -758,6 +830,7 @@ def CloudSQLSource(
         db=db,
         user=user,
         password=password,
+        engine_args=engine_args,
     )
 
 
