@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING, Any, Dict, Optional
 
 from chalk.integrations.named import load_integration_variable
 from chalk.sql._internal.sql_source import BaseSQLSource
@@ -20,14 +20,19 @@ class BigQuerySourceImpl(BaseSQLSource):
         location: Optional[str] = None,
         credentials_base64: Optional[str] = None,
         credentials_path: Optional[str] = None,
+        engine_args: Optional[Dict[str, Any]] = None,
     ):
         try:
             import sqlalchemy_bigquery
         except ModuleNotFoundError:
             raise missing_dependency_exception("chalkpy[bigquery]")
         del sqlalchemy_bigquery  # unused
+        if engine_args is None:
+            engine_args = {}
+        engine_args.setdefault("pool_size", 20)
+        engine_args.setdefault("max_overflow", 60)
 
-        BaseSQLSource.__init__(self, name=name)
+        BaseSQLSource.__init__(self, name=name, engine_args=engine_args)
         self.location = location or load_integration_variable(integration_name=name, name="BQ_LOCATION")
         self.dataset = dataset or load_integration_variable(integration_name=name, name="BQ_DATASET")
         self.project = project or load_integration_variable(integration_name=name, name="BQ_PROJECT")
