@@ -1,7 +1,7 @@
 import os
 import sys
 from pathlib import Path
-from typing import Optional
+from typing import List, Optional
 
 
 def get_classpath(x: object) -> Path:
@@ -19,3 +19,16 @@ def get_directory_root() -> Optional[Path]:
             # This is '/'
             return None
         current = current.parent
+
+
+def _search_recursively_for_file(base: Path, filename: str) -> List[Path]:
+    ans = []
+    assert base.is_dir()
+    while True:
+        filepath = base / filename
+        if filepath.exists():
+            ans.append(filepath)
+        parent = base.parent
+        if parent == base:
+            return ans
+        base = parent
